﻿using System;
using System.IO;
using System.Linq;
using System.Windows.Forms;

namespace SimpleGameMaker
{
    public partial class Form1 : Form
    {
        public const int COL_NUM = 0;
        public const int COL_TIME = 1;
        public const int COL_ALIEN = 2;
        public const int COL_IMAGE = 3;
        public const int COL_WAVE = 4;
        public const int COL_MESSAGE = 5;
        public const int COL_COMMENT = 6;

        /// <summary>
        /// Path to the appplication directory
        /// </summary>
        public static string AppPath = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location) + @"\";

        public Form1()
        {
            InitializeComponent();

            this.WindowState = FormWindowState.Maximized;
            
            AddDebugMessage("Debug Log Window");
            
            // Add the alien images to the selection box
            for (int j = 0; j < this.imageListAliens.Images.Count; j++)
            {

                ListViewItem item = new ListViewItem();
                item.ImageIndex = j;
                this.listViewPreview.Items.Add(item);
            }
        }

        /// <summary>
        ///  Load a file from CSV file to a grid view.
        /// </summary>
        /// <param name="sender">Sending object</param>
        /// <param name="e">Event arguments</param>
        private void buttonClear_Click(object sender, EventArgs e)
        {
            this.dataGridViewFull.Rows.Clear();
            this.AddDebugMessage("Table Cleared");
        }

        /// <summary>
        ///  Load a file from CSV file to a grid view.
        /// </summary>
        /// <param name="sender">Sending object</param>
        /// <param name="e">Event arguments</param>
        private void buttonLoad_Click(object sender, EventArgs e)
        {
            this.LoadTable(this.dataGridViewFull, "Table loaded ");
        }

        /// <summary>
        /// Load a file from CSV file to a grid view.
        /// </summary>
        /// <param name="theGrid">The data grid to populate</param> 
        public void LoadTable(DataGridView theGrid, string theMessage)
        {
            // Show the open dialog so the user can select a file
            OpenFileDialog fileDialog = new OpenFileDialog();
            fileDialog.Title = "Select file";
            fileDialog.InitialDirectory = Form1.AppPath;
            fileDialog.Filter = "Comma separated values (*.csv)|*.csv|All files (*.*)|*.*";
            fileDialog.FilterIndex = 1;
            fileDialog.RestoreDirectory = true;
            if (fileDialog.ShowDialog() == DialogResult.OK)
            {
                this.ImportFile(fileDialog.FileName, theGrid);
                textBoxLevelName.Text = Path.GetFileNameWithoutExtension(fileDialog.FileName);
                this.AddDebugMessage(theMessage + fileDialog.FileName);
            }
        }

        /// <summary>
        ///  Import a CSV file into a grid view 
        /// </summary>
        /// <param name="sFileName">The file to load</param>
        /// <param name="theGridToPopulate">The data grid to populate</param>
        /// <returns></returns>
        private bool ImportFile(String sFileName, DataGridView theGridToPopulate)
        {
            try
            {
                String textLine = string.Empty;
                String[] splitLine;

                // clear the grid view
                theGridToPopulate.Rows.Clear();

                if (System.IO.File.Exists(sFileName))
                {
                    using (System.IO.StreamReader objReader = new System.IO.StreamReader(sFileName))
                    {
                        textLine = objReader.ReadLine(); // Ignore the first line
                        do
                        {
                            textLine = objReader.ReadLine();
                            if (textLine != "")
                            {
                                splitLine = textLine.Split('\t');
                                if (theGridToPopulate.Name == dataGridViewFull.Name)
                                {
                                    int p = Convert.ToInt16(splitLine[2]);
                                    int n = 0;
                                    n = theGridToPopulate.Rows.Add(splitLine[0], splitLine[1], splitLine[2], imageListAliens.Images[p], splitLine[4], splitLine[5], splitLine[6], splitLine[7]);
                                    dataGridViewFull.Rows[n].Height = 70;
                                }
                                else
                                {
                                    theGridToPopulate.Rows.Add(splitLine[0], splitLine[1]);
                                }
                            }
                        } while (objReader.Peek() != -1);
                    }
                }
                this.AddDebugMessage("Table Loaded " + sFileName);
                return true;
            }
            catch (Exception ex)
            {
                this.AddDebugMessage(ex.Message.ToString());
                return false;
            }
        }

        /// <summary>
        /// Save the grid view to a csv file.
        /// </summary>
        /// <param name="sender">Sending object</param>
        /// <param name="e">Event arguments</param>
        private void buttonSave_Click(object sender, EventArgs e)
        {
            if (dataGridViewFull.RowCount > 0)
            {
                this.SaveTable(this.dataGridViewFull, "Table saved ", textBoxLevelName.Text);
            }
            else
            {
                this.AddDebugMessage("No level to save ?");
            }
        }

        /// <summary>
        /// Save a grid view to a file shows save file dialogue
        /// </summary>
        /// <param name="theGrid">The grid to save</param> 
        public void SaveTable(DataGridView theGrid, string theMessage, string theFile)
        {
            SaveFileDialog savefile = new SaveFileDialog();
            savefile.InitialDirectory = Form1.AppPath;

            // set a default file name
            savefile.FileName = theFile + ".csv";

            // set filters - this can be done in properties as well
            savefile.Filter = "Comma separated values (*.csv)|*.csv|All files (*.*)|*.*";

            if (savefile.ShowDialog() == DialogResult.OK)
            {
                this.ExportFile(savefile.FileName, theGrid, theMessage);
            }
        }

        /// <summary>
        /// Export a grid view to a CSV file
        /// </summary>
        /// <param name="saveFilename">The filename to save to</param> 
        /// <param name="theGrid">DataGridView we are saving</param> 
        private void ExportFile(string saveFilename, DataGridView theGrid, string theMessage)
        {
            // create one file gridview.csv in writing mode using streamwriter
            using (StreamWriter streamGrid = new StreamWriter(saveFilename))
            {
                // now add the gridview header in csv file suffix with "," delimeter except last one
                for (int i = 0; i < theGrid.Columns.Count; i++)
                {
                    streamGrid.Write(theGrid.Columns[i].HeaderText);
                    if (i != theGrid.Columns.Count)
                    {
                        streamGrid.Write("\t");
                    }
                }

                // add new line
                streamGrid.Write(streamGrid.NewLine);

                // iterate through all the rows within the gridview
                foreach (DataGridViewRow dr in theGrid.Rows)
                {
                    // iterate through all colums of specific row
                    for (int i = 0; i < theGrid.Columns.Count; i++)
                    {
                        string temp;
                        if ((dr.Cells[i].Value != null) && (dr.Cells[i].GetType() != typeof(DataGridViewImageCell)))
                        {
                            temp = dr.Cells[i].Value.ToString();
                        }
                        else
                        {
                            temp = string.Empty;
                        }

                        temp = temp.Replace('\r', ' ');
                        temp = temp.Replace('\n', ' ');

                        // write particular cell to csv file
                        streamGrid.Write(temp);
                        if (i != theGrid.Columns.Count)
                        {
                            streamGrid.Write("\t");
                        }
                    }

                    // write new line
                    streamGrid.Write(streamGrid.NewLine);
                }

                // flush from the buffers.
                streamGrid.Flush();
            }
            this.AddDebugMessage(theMessage + saveFilename);
        }

        /// <summary>
        /// Copy the clipboard table to CSV
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonClipboardTable_Click(object sender, EventArgs e)
        {
            Clipboard.SetText(ClipBoardCSV(dataGridViewFull));
        }

        /// <summary>
        /// Copy the CSV to clip board
        /// </summary>
        /// <param name="theGridToPopulate"></param>
        /// <returns></returns>
        public string ClipBoardCSV(DataGridView theGridToPopulate)
        {
            try
            {
                int[] allowedColumns = { 1, 2, 4, 5, 6, 7 };

                string Output = "-- Columns ";
                // now add the gridview header in csv file suffix with "," delimeter except last one
                for (int i = 0; i < theGridToPopulate.Columns.Count; i++)
                {
                    if (allowedColumns.Contains(i))
                    {
                        Output += theGridToPopulate.Columns[i].HeaderText;
                        if (i != theGridToPopulate.Columns.Count)
                        {
                            Output += ",";
                        }
                    }
                }
                Output += "\r\n" + textBoxLevelName.Text + " = {\r\n";
                bool bDone = false;
                // iterate through all the rows within the gridview
                foreach (DataGridViewRow dr in theGridToPopulate.Rows)
                {
                    // iterate through all colums of specific row
                    for (int i = 0; i < theGridToPopulate.Columns.Count; i++)
                    {

                        if (allowedColumns.Contains(i))
                        {
                            if (i != COL_COMMENT)
                            {
                                String sTemp;
                                if (dr.Cells[i].Value != null)
                                    sTemp = dr.Cells[i].Value.ToString();
                                else
                                {
                                    bDone = true;
                                    break;
                                }

                                sTemp = sTemp.Replace('\r', ' ');
                                sTemp = sTemp.Replace('\n', ' ');

                                // write particular cell to csv file
                                if (i != COL_IMAGE)
                                {
                                    // Messages replace with correct quotes
                                    if (i == COL_MESSAGE)
                                    {
                                        if (sTemp.Length > 0)
                                            Output += "\"" + sTemp.Replace("\"", "\'") + "\"";
                                        else
                                            Output += "0";
                                    }
                                    else
                                    {
                                        Output += sTemp;
                                    }
                                }
                                if (i != theGridToPopulate.Columns.Count)
                                {
                                    Output += ",";
                                }
                            }
                            else
                            {
                                String sTemp;
                                sTemp = dr.Cells[i].Value.ToString();
                                if (sTemp.Length > 0)
                                {
                                    Output += " -- " + sTemp;
                                }
                            }
                        }
                    }
                    if (bDone)
                    {
                        break;
                    }
                    Output += "\r\n";
                }
                Output += "}\r\n";
                AddDebugMessage("\r\nLevel Created \r\n\n " + Output);
                return Output;
            }
            catch (Exception ex)
            {
                this.AddDebugMessage(ex.Message.ToString());
            }
            return (string.Empty);
        }


        /// <summary>
        /// Create a duplicate of the currently selected row
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonDuplicate_Click(object sender, EventArgs e)
        {
            try
            {
                if (dataGridViewFull.RowCount > 0)
                {
                    if (dataGridViewFull.SelectedRows.Count > 0)
                    {
                        int rowCount = dataGridViewFull.Rows.Count;
                        int i = dataGridViewFull.SelectedCells[0].OwningRow.Index;
                        DataGridViewRow nextRow = dataGridViewFull.Rows[i];
                        int n = dataGridViewFull.Rows.Add();
                        for (Int32 index = 0; index < nextRow.Cells.Count; index++)
                        {
                            dataGridViewFull.Rows[n].Cells[index].Value = nextRow.Cells[index].Value;
                        }
                        dataGridViewFull.Rows[n].Height = 70;
                        dataGridViewFull.ClearSelection();
                    }
                }
            }
            catch (Exception ex)
            {
                this.AddDebugMessage(ex.Message.ToString());
            }
        }

        /// <summary>
        /// Delete the selected row(s)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonDeleteRow_Click(object sender, EventArgs e)
        {
            try
            {
                foreach (DataGridViewCell oneCell in dataGridViewFull.SelectedCells)
                {
                    if (oneCell.Selected)
                        dataGridViewFull.Rows.RemoveAt(oneCell.RowIndex);
                }
            }
            catch (Exception ex)
            {
                this.AddDebugMessage(ex.Message.ToString());
            }
        }

        /// <summary>
        /// Add the wave row with some checking
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonAddRow_Click(object sender, EventArgs e)
        {
            try
            {
                int imageValue = Convert.ToInt32(textBoxAlienType.Text);
                if ((imageValue > imageListAliens.Images.Count) && (imageValue != 99))
                {
                    throw new System.ArgumentException("Value not in range ", "Alien Type");
                }
                int n = this.dataGridViewFull.Rows.Add();
                this.dataGridViewFull.Rows[n].Height = 70;
                this.dataGridViewFull.Rows[n].Cells[COL_NUM].Value = this.dataGridViewFull.RowCount;
                this.dataGridViewFull.Rows[n].Cells[COL_TIME].Value = textBoxTime.Text;
                this.dataGridViewFull.Rows[n].Cells[COL_ALIEN].Value = textBoxAlienType.Text;
                this.dataGridViewFull.Rows[n].Cells[COL_WAVE].Value = textBoxWaveType.Text;
                this.dataGridViewFull.Rows[n].Cells[COL_MESSAGE].Value = textBoxDescription.Text;
                this.dataGridViewFull.Rows[n].Cells[COL_COMMENT].Value = textBoxComment.Text;
                this.dataGridViewFull.Rows[n].Cells[COL_IMAGE].Value = imageListAliens.Images[Convert.ToInt32(textBoxAlienType.Text)];
                textBoxTime.Text = (Convert.ToInt32(textBoxTime.Text) + Convert.ToInt32(textBoxIncrement.Text)).ToString();
            }
            catch (Exception ex)
            {
                this.AddDebugMessage(ex.Message.ToString());
            }
        }

        /// <summary>
        /// Add a debug message to show the user the state of things
        /// </summary>
        /// <param name="debugMessage"></param>
        public void AddDebugMessage(string debugMessage)
        {
            int maxLines = 40;
            string[] arrayString = textBoxLog.Text.Split('\n');

            if (arrayString.Length > maxLines)
            {
                textBoxLog.Text = string.Join("\n", arrayString.Skip(arrayString.Length - maxLines));
            }

            textBoxLog.Text += debugMessage + "\r\n";
        }

        /// <summary>
        /// When the user clicks an Alien in the preview update the choice textbox to show that Alien
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listViewPreview_Click(object sender, EventArgs e)
        {
            try
            {
                if (listViewPreview.SelectedItems.Count > 0)
                {
                    textBoxAlienType.Text = listViewPreview.Items.IndexOf(listViewPreview.SelectedItems[0]).ToString();
                }
            }
            catch (Exception ex)
            {
                this.AddDebugMessage(ex.Message.ToString());
            }
        }

        /// <summary>
        /// Update the alien image list
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void textBoxAlienType_TextChanged(object sender, EventArgs e)
        {
            try
            {
                if (textBoxAlienType.Text.Length > 0)
                {
                    int p = Convert.ToInt16(textBoxAlienType.Text);
                    pictureBoxAlien.Image = imageListAliens.Images[p];
                }
            }
            catch
            { }
        }

        /// <summary>
        /// Move a row to the top
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonRowTop_Click(object sender, EventArgs e)
        {
            try
            {
                DataGridViewSelectedRowCollection dgvsrc = dataGridViewFull.SelectedRows; // get the collection of selected rows
                if (dgvsrc.Count > 0)
                {
                    int index = dataGridViewFull.SelectedRows[0].Index;//Get the index of the currently selected row
                    if (index > 0) // if the line is not the first line
                    {
                        DataGridViewRow dgvr = dataGridViewFull.Rows[index];//Get the previous row of the selected row
                        dataGridViewFull.Rows.RemoveAt(index);//Delete the previous row of the original selected row
                        dataGridViewFull.Rows.Insert(0, dgvr);//Insert the previous line of the selected line after the selected line                       
                        for (int i = 0; i < dataGridViewFull.Rows.Count; i++)//Select the row after the move
                        {
                            if (i != 0)
                            {
                                dataGridViewFull.Rows[i].Selected = false;
                            }
                            else
                            {
                                dataGridViewFull.Rows[i].Selected = true;
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                this.AddDebugMessage(ex.Message.ToString());
            }
        }


        /// <summary>
        /// Move a row to the bottom
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonRowBottom_Click(object sender, EventArgs e)
        {

            try
            {
                DataGridViewSelectedRowCollection dgvsrc = dataGridViewFull.SelectedRows; // get the collection of selected rows
                if (dgvsrc.Count > 0)
                {
                    int index = dataGridViewFull.SelectedRows[0].Index;//Get the index of the currently selected row
                    if (index < dataGridViewFull.Rows.Count - 1) // if the line is not the first line
                    {
                        DataGridViewRow dgvr = dataGridViewFull.Rows[index];//Get the previous row of the selected row
                        dataGridViewFull.Rows.RemoveAt(index);//Delete the previous row of the original selected row
                        int nCount = dataGridViewFull.Rows.Count;
                        dataGridViewFull.Rows.Insert(nCount, dgvr);//Insert the previous line of the selected line after the selected line
                        for (int i = 0; i < dataGridViewFull.Rows.Count; i++)//Select the row after the move
                        {
                            if (i != dataGridViewFull.Rows.Count - 1)
                            {
                                dataGridViewFull.Rows[i].Selected = false;
                            }
                            else
                            {
                                dataGridViewFull.Rows[i].Selected = true;
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                this.AddDebugMessage(ex.Message.ToString());
            }
        }

        /// <summary>
        /// Move the selected rows up one
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonRowUp_Click(object sender, EventArgs e)
        {
            try
            {
                if (dataGridViewFull.RowCount > 0)
                {
                    if (dataGridViewFull.SelectedRows.Count > 0)
                    {
                        int rowCount = dataGridViewFull.Rows.Count;
                        int index = dataGridViewFull.SelectedCells[0].OwningRow.Index;

                        if (index == 0)
                        {
                            return;
                        }
                        DataGridViewRowCollection rows = dataGridViewFull.Rows;

                        // remove the previous row and add it behind the selected row.
                        DataGridViewRow prevRow = rows[index - 1];
                        rows.Remove(prevRow);
                        prevRow.Frozen = false;
                        rows.Insert(index, prevRow);
                        dataGridViewFull.ClearSelection();
                        dataGridViewFull.Rows[index - 1].Selected = true;
                    }
                }
            }
            catch (Exception ex)
            {
                this.AddDebugMessage(ex.Message.ToString());
            }
        }
        /// <summary>
        /// Move the selected rows down one
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonRowDown_Click(object sender, EventArgs e)
        {
            try
            {
                if (dataGridViewFull.RowCount > 0)
                {
                    if (dataGridViewFull.SelectedRows.Count > 0)
                    {
                        int rowCount = dataGridViewFull.Rows.Count;
                        int index = dataGridViewFull.SelectedCells[0].OwningRow.Index;

                        if (index == (rowCount - 1)) // include the header row
                        {
                            return;
                        }
                        DataGridViewRowCollection rows = dataGridViewFull.Rows;

                        // remove the next row and add it in front of the selected row.
                        DataGridViewRow nextRow = rows[index + 1];
                        rows.Remove(nextRow);
                        nextRow.Frozen = false;
                        rows.Insert(index, nextRow);
                        dataGridViewFull.ClearSelection();
                        dataGridViewFull.Rows[index + 1].Selected = true;
                    }
                }
            }
            catch (Exception ex)
            {
                this.AddDebugMessage(ex.Message.ToString());
            }
        }

        /// <summary>
        /// Setup the wave type of the alien
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void radioButtonWave1_Click(object sender, EventArgs e)
        {
            RadioButton theRadio = (RadioButton)sender;
            if (!(theRadio is null))
            {
                textBoxWaveType.Text = theRadio.Tag.ToString();
            }
        }

        /// <summary>
        /// Clear the log file
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonClearDebug_Click(object sender, EventArgs e)
        {
            textBoxLog.Text = "";
        }
    }
}
